@@ -1,3 +1,4 @@
+
 /* This file is part of the KDE project
  * Copyright (C) 2012 Arjen Hiemstra <ahiemstra@heimr.nl>
  *
@@ -54,6 +55,7 @@ public:
         , currentShortcut(0)
         , tabletPressEvent(0)
         , setMirrorMode(false)
+        , fixedAction(false)
     { }
 
     void match(QEvent *event);
@@ -79,6 +81,7 @@ public:
     QTabletEvent *tabletPressEvent;
 
     bool setMirrorMode;
+    bool fixedAction;
 };
 
 KisInputManager::KisInputManager(KisCanvas2 *canvas, KoToolProxy *proxy)
@@ -102,6 +105,9 @@ KisInputManager::KisInputManager(KisCanvas2 *canvas, KoToolProxy *proxy)
     d->canvas->view()->actionCollection()->addAction("set_mirror_axis", setMirrorAxis);
     setMirrorAxis->setShortcut(QKeySequence("Shift+r"));
     connect(setMirrorAxis, SIGNAL(triggered(bool)), SLOT(setMirrorAxis()));
+
+    connect(KoToolManager::instance(), SIGNAL(changedTool(KoCanvasController*,int)),
+            SLOT(slotToolChanged()));
 }
 
 KisInputManager::~KisInputManager()
@@ -151,7 +157,7 @@ bool KisInputManager::eventFilter(QObject* object, QEvent* event)
             if (d->currentAction) { //If we are currently performing an action, we only update the state of that action and shortcut.
                 d->currentShortcut->match(event);
 
-                if (d->currentShortcut->matchLevel() == KisShortcut::NoMatch) {
+                if (d->currentShortcut->matchLevel() == KisShortcut::NoMatch && !d->fixedAction) {
                     d->clearState();
                     break;
                 }
@@ -262,6 +268,21 @@ void KisInputManager::setMirrorAxis()
     QApplication::setOverrideCursor(Qt::CrossCursor);
 }
 
+void KisInputManager::slotToolChanged()
+{
+    QString toolId = KoToolManager::instance()->activeToolId();
+    if (toolId == "ArtisticTextToolFactoryID" || toolId == "TextToolFactory_ID") {
+        d->fixedAction = true;
+        if (!d->currentAction) {
+            d->currentShortcut = d->shortcuts.at(0);
+            d->currentAction = d->currentShortcut->action();
+            d->currentAction->begin(d->currentShortcut->shortcutIndex());
+        }
+    } else {
+        d->fixedAction = false;
+    }
+}
+
 QPointF KisInputManager::widgetToPixel(const QPointF& position)
 {
     QPointF pixel = QPointF(position.x() + 0.5f, position.y() + 0.5f);
@@ -270,6 +291,9 @@ QPointF KisInputManager::widgetToPixel(const QPointF& position)
 
 void KisInputManager::Private::match(QEvent* event)
 {
+    if (fixedAction) {
+        return;
+    }
     //Go through all possible shortcuts and update their state.
     foreach (KisShortcut* shortcut, potentialShortcuts) {
         shortcut->match(event);
@@ -316,9 +340,12 @@ void KisInputManager::Private::setupActions()
     KisAbstractInputAction* action = new KisToolInvocationAction(q);
     actions.append(action);
 
-    KisShortcut* shortcut = createShortcut(action, 0);
+    KisShortcut* shortcut = createShortcut(action, KisToolInvocationAction::ActivateShortcut);
     shortcut->setButtons(QList<Qt::MouseButton>() << Qt::LeftButton);
 
+    shortcut = createShortcut(action, KisToolInvocationAction::ConfirmShortcut);
+    shortcut->setKeys(QList<Qt::Key>() << Qt::Key_Return);
+
     action = new KisAlternateInvocationAction(q);
     actions.append(action);
 
@@ -340,7 +367,11 @@ void KisInputManager::Private::setupActions()
     shortcut->setKeys(QList<Qt::Key>() << Qt::Key_Space);
 
     shortcut = createShortcut(action, KisPanAction::PanToggleShortcut);
+#if QT_VERSION >= 0x040700
     shortcut->setButtons(QList<Qt::MouseButton>() << Qt::MiddleButton);
+#else
+    shortcut->setButtons(QList<Qt::MouseButton>() << Qt::MidButton);
+#endif
 
     shortcut = createShortcut(action, KisPanAction::PanLeftShortcut);
     shortcut->setKeys(QList<Qt::Key>() << Qt::Key_Left);
@@ -359,7 +390,11 @@ void KisInputManager::Private::setupActions()
 
     shortcut = createShortcut(action, KisRotateCanvasAction::RotateToggleShortcut);
     shortcut->setKeys(QList<Qt::Key>() << Qt::Key_Shift);
+#if QT_VERSION >= 0x040700
     shortcut->setButtons(QList<Qt::MouseButton>() << Qt::MiddleButton);
+#else
+    shortcut->setButtons(QList<Qt::MouseButton>() << Qt::MidButton);
+#endif
 
     shortcut = createShortcut(action, KisRotateCanvasAction::RotateLeftShortcut);
     shortcut->setKeys(QList<Qt::Key>() << Qt::Key_4);
@@ -378,7 +413,11 @@ void KisInputManager::Private::setupActions()
 
     shortcut = createShortcut(action, KisZoomAction::ZoomToggleShortcut);
     shortcut->setKeys(QList<Qt::Key>() << Qt::Key_Control);
+#if QT_VERSION >= 0x040700
     shortcut->setButtons(QList<Qt::MouseButton>() << Qt::MiddleButton);
+#else
+    shortcut->setButtons(QList<Qt::MouseButton>() << Qt::MidButton);
+#endif
 
     shortcut = createShortcut(action, KisZoomAction::ZoomInShortcut);
     shortcut->setWheel(KisShortcut::WheelUp);
@@ -401,6 +440,9 @@ void KisInputManager::Private::setupActions()
 
     shortcut = createShortcut(action, 0);
     shortcut->setButtons(QList<Qt::MouseButton>() << Qt::RightButton);
+
+    shortcut = createShortcut(action, 0);
+    shortcut->setKeys(QList<Qt::Key>() << Qt::Key_F);
 }
 
 KisShortcut* KisInputManager::Private::createShortcut(KisAbstractInputAction* action, int index)
@@ -415,6 +457,10 @@ KisShortcut* KisInputManager::Private::createShortcut(KisAbstractInputAction* ac
 
 void KisInputManager::Private::clearState()
 {
+    if (fixedAction) {
+        return;
+    }
+
     if (currentShortcut) {
         currentAction->end();
         currentAction = 0;
